;*=====================================================================*/
;*    serrano/prgm/project/bigloo/fthread/src/fair-reader.scm          */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Mon Feb 11 12:00:29 2002                          */
;*    Last change :  Thu Jul 11 10:37:22 2002 (serrano)                */
;*    Copyright   :  2002 Manuel Serrano                               */
;*    -------------------------------------------------------------    */
;*    Fair readers, that is cooperative readers.                       */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __ft_reader

   (import __ft_types
	   __ft_scheduler
	   __ft_%thread)

   (export (make-fair-reader ::input-port ::int ::obj)
	   (make-fair-match ::input-port ::bstring ::obj)
	   (make-fair-memq ::input-port ::pair ::obj)))

;*---------------------------------------------------------------------*/
;*    make-fair-reader ...                                             */
;*    -------------------------------------------------------------    */
;*    This function is the non blocking input main entry point. A      */
;*    fair thread asks READER-AWAIT to be notified when N chars        */
;*    read on PORT are available. When this condition is meet, the     */
;*    event EVENT is signaled.                                         */
;*---------------------------------------------------------------------*/
(define (make-fair-reader port size event)
   ;; The STRING must be allocated before the tread because the
   ;; thread must not allocate any Bigloo ressource.
   ;; CELL, the cons that is broadcast when the input is completed
   ;; has to be pre-allocated for the same reason.
   (let ((str (make-string size))
	 (cell (cons #unspecified #unspecified)))
      (lambda (scdl)
	 (lambda ()
	    (let loop ((n 0))
	       (if (=fx n size)
		   (begin
		      (set-car! cell str)
		      (set-cdr! cell #f)
		      (scheduler-broadcast scdl event cell))
		   (let ((c (read-char port)))
		      (if (eof-object? c)
			  (begin
			     (set-car! cell (substring str 0 n))
			     (set-cdr! cell #t)
			     (scheduler-broadcast scdl event cell))
			  (begin
			     (string-set! str n c)
			     (loop (+fx n 1)))))))))))

;*---------------------------------------------------------------------*/
;*    string-double ...                                                */
;*---------------------------------------------------------------------*/
(define (string-double str)
   (let* ((size (string-length str))
	  (new-size (*fx size 2))
	  (new-buffer (make-string new-size)))
      (blit-string! new-buffer 0 str 0 size)
      new-buffer))

;*---------------------------------------------------------------------*/
;*    make-fair-match ...                                              */
;*    -------------------------------------------------------------    */
;*    Read characters until a user pattern is found. Then returns      */
;*    a Java byte array representing the buffer.                       */
;*---------------------------------------------------------------------*/
(define (make-fair-match port match event)
   (let ((cell (cons #f #f)))
      (lambda (scdl)
	 (lambda ()
	    (define (match-found? n buffer len)
	       (if (>= n len)
		   (let loop ((r1 0)
			      (r2 (-fx n len)))
		      (cond
			 ((=fx r2 n)
			  #t)
			 ((char=? (string-ref buffer r2) (string-ref match r1))
			  (loop (+fx r1 1) (+fx r2 1)))
			 (else
			  #f)))
		   #f))
	    (let loop ((n 0)
		       (size 64)
		       (buffer (make-string 64)))
	       (if (=fx n size)
		   (loop n
			 (*fx size 2)
			 (string-double buffer))
		   (let ((c (read-char port)))
		      (if (eof-object? c)
			  (begin
			     (set-car! cell buffer)
			     (set-cdr! cell #t)
			     (scheduler-broadcast scdl event cell))
			  (begin
			     (string-set! buffer n c)
			     (if (match-found? n buffer size)
				 (let ((str (substring buffer 0 (+fx n 1))))
				    (set-car! cell str)
				    (set-cdr! cell #f)
				    (scheduler-broadcast scdl event cell))
				 (loop (+fx n 1)
				       size
				       buffer)))))))))))

;*---------------------------------------------------------------------*/
;*    make-fair-memq ...                                               */
;*    -------------------------------------------------------------    */
;*    Reads everything until we find a character in the SET.           */
;*    The returned strings *contains* that character.                  */
;*---------------------------------------------------------------------*/
(define (make-fair-memq port match event)
   (let ((cell (cons #f #f)))
      (lambda (scdl)
	 (lambda ()
	    (let loop ((n 0)
		       (size 64)
		       (buffer (make-string 64)))
	       (if (=fx n size)
		   (loop n
			 (*fx size 2)
			 (string-double buffer))
		   (let ((c (read-char port)))
		      (if (eof-object? c)
			  (begin
			     (set-car! cell buffer)
			     (set-cdr! cell #t)
			     (scheduler-broadcast scdl event cell))
			  (begin
			     (string-set! buffer n c)
			     (if (memq c match)
				 (let ((s (substring buffer 0 (+fx n 1))))
				    (set-car! cell s)
				    (set-cdr! cell #f)
				    (scheduler-broadcast scdl event cell))
				 (loop (+fx n 1)
				       size
				       buffer)))))))))))
