;*=====================================================================*/
;*    serrano/prgm/project/bigloo/fthread/src/signal.scm               */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Wed Feb 13 14:36:25 2002                          */
;*    Last change :  Thu Jul 11 10:50:18 2002 (serrano)                */
;*    Copyright   :  2002 Manuel Serrano                               */
;*    -------------------------------------------------------------    */
;*    Fair thread signal handling                                      */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __ft_signal
   
   (import __ft_types
	   __ft_env
	   __ft_%thread)
   
   (export (class %signal
	      (id::obj read-only)
	      (vals::pair-nil (default '()))
	      (stamp::int (default -1))
	      (threads::pair-nil (default '())))
	   
	   (signal-lookup ::obj ::pair)
	   (signal-emit ::obj ::obj ::pair)
	   (signal-values ::obj ::pair)
	   (signal-register-thread! ::obj ::pair ::thread)
	   (signal-unregister-thread! ::obj ::pair ::thread)))

;*---------------------------------------------------------------------*/
;*    signal-lookup ...                                                */
;*---------------------------------------------------------------------*/
(define (signal-lookup id envs)
   (define (%signal-lookup id env)
      (let ((s (ftenv-lookup env id)))
	 (if (%signal? s)
	     (with-access::%signal s (id stamp)
		(if (=fx stamp (ftenv-instant env)) s #f))
	     #f)))
   (let loop ((envs envs))
      (if (ftenv-handles? (car envs) id)
	  (%signal-lookup id (car envs))
	  (loop (cdr envs)))))

;*---------------------------------------------------------------------*/
;*    signal-awake-threads ...                                         */
;*---------------------------------------------------------------------*/
(define (signal-awake-threads s::%signal)
   (with-access::%signal s (threads)
      (let ((th threads))
	 (set! threads '())
	 (if (> *thread-debug* 1)
	     (display* "*** signal-awake!: " (%signal-id s)))
	 (for-each (lambda (thread)
		      (if (> *thread-debug* 1) (display* " " thread))
		      (thread-set-runnable! thread))
		   th)
	 (if (> *thread-debug* 1) (newline)))))

;*---------------------------------------------------------------------*/
;*    signal-emit ...                                                  */
;*---------------------------------------------------------------------*/
(define (signal-emit id value envs)
   (define (%signal-emit id value env)
      (with-access::ftenv env (instant)
	 (let ((o (ftenv-lookup env id)))
	    (if (> *thread-debug* 2)
		(print "*** signal-emit: " id " " (find-runtime-type o)))
	    (if (%signal? o)
		(with-access::%signal o (vals stamp threads)
		   (set! stamp instant)
		   (set! vals (cons value vals))
		   (signal-awake-threads o))
		(ftenv-bind! env
			     id
			     (instantiate::%signal
				(id id)
				(vals (list value))
				(stamp instant)))))))
   (let loop ((envs envs))
      (if (ftenv-handles? (car envs) id)
	  (%signal-emit id value (car envs))
	  (loop (cdr envs)))))

;*---------------------------------------------------------------------*/
;*    signal-values ...                                                */
;*---------------------------------------------------------------------*/
(define (signal-values e envs)
   (let ((o (signal-lookup e envs)))
      (if (%signal? o)
	  (%signal-vals o)
	  #unspecified)))

;*---------------------------------------------------------------------*/
;*    signal-register-thread! ...                                      */
;*---------------------------------------------------------------------*/
(define (signal-register-thread! id envs thread)
   (if (> *thread-debug* 1)
       (print "+++ signal-register-thread!: " id " " thread))
   (define (%signal-register-thread! id env thread)
      (let ((o (ftenv-lookup env id)))
	 (if (%signal? o)
	     ;; the signal already exists, we add thread to the threads list
	     (with-access::%signal o (threads)
		(set! threads (cons thread threads)))
	     ;; the signal does not exist, we bind it
	     (let ((new (instantiate::%signal
			   (id id)
			   (vals '())
			   (stamp -1)
			   (threads (list thread)))))
		(ftenv-bind! env id new)))))
   (let loop ((envs envs))
      (if (ftenv-handles? (car envs) id)
	  (%signal-register-thread! id (car envs) thread)
	  (loop (cdr envs)))))

;*---------------------------------------------------------------------*/
;*    signal-unregister-thread! ...                                    */
;*---------------------------------------------------------------------*/
(define (signal-unregister-thread! id envs thread)
   (if (> *thread-debug* 1)
       (print "+++ signal-unregister-thread!: " id " " thread))
   (define (%signal-unregister-thread! id env thread)
      (let ((o (ftenv-lookup env id)))
	 ;; the signal already exists, we add thread to the threads list
	 (with-access::%signal o (threads)
	    (set! threads (remq! thread threads)))))
   (let loop ((envs envs))
      (if (ftenv-handles? (car envs) id)
	  (%signal-unregister-thread! id (car envs) thread)
	  (loop (cdr envs)))))
