/*=====================================================================*/
/*    serrano/prgm/project/bigloo/runtime/Clib/cthread.c               */
/*    -------------------------------------------------------------    */
/*    Author      :  Manuel Serrano                                    */
/*    Creation    :  Tue Feb 26 15:20:32 2002                          */
/*    Last change :  Wed Aug  7 11:13:36 2002 (serrano)                */
/*    Copyright   :  2002 Manuel Serrano                               */
/*    -------------------------------------------------------------    */
/*    This file contains the C definitions of all variables and        */
/*    functions that are thread specific.                              */
/*=====================================================================*/
#include <bigloo.h>

/*---------------------------------------------------------------------*/
/*    Ecxeption stack                                                  */
/*---------------------------------------------------------------------*/
static obj_t exitd_top = BFALSE;
static obj_t exitd_stamp = BINT( 0 );

/*---------------------------------------------------------------------*/
/*    dynamic-wind before thunk linking variable.                      */
/*---------------------------------------------------------------------*/
static struct befored *befored_top = 0L;

/*---------------------------------------------------------------------*/
/*    Dynamic ports                                                    */
/*---------------------------------------------------------------------*/
static obj_t current_output_port, current_input_port, current_error_port;

/*---------------------------------------------------------------------*/
/*    Multiple values                                                  */
/*---------------------------------------------------------------------*/
static int mvalues_number = 1;
static obj_t mvalues[ 4 ] = { BUNSPEC, BUNSPEC, BUNSPEC, BUNSPEC };

/*---------------------------------------------------------------------*/
/*    Stack bottom mark (for call/cc) ...                              */
/*---------------------------------------------------------------------*/
static char *stack_bottom;

/*---------------------------------------------------------------------*/
/*    static char *                                                    */
/*    get_stack_bottom ...                                             */
/*---------------------------------------------------------------------*/
static char *
get_stack_bottom() {
   return stack_bottom;
}

/*---------------------------------------------------------------------*/
/*    static void                                                      */
/*    set_stack_bottom ...                                             */
/*---------------------------------------------------------------------*/
static void
set_stack_bottom( char *val ) {
   stack_bottom = val;
}

/*---------------------------------------------------------------------*/
/*    static obj_t                                                     */
/*    get_exitd_top ...                                                */
/*---------------------------------------------------------------------*/
static obj_t
get_exitd_top() {
   return exitd_top;
}

/*---------------------------------------------------------------------*/
/*    static obj_t                                                     */
/*    set_exitd_top ...                                                */
/*---------------------------------------------------------------------*/
static obj_t
set_exitd_top( obj_t val ) {
   exitd_top = val;
   return BUNSPEC;
}

/*---------------------------------------------------------------------*/
/*    obj_t                                                            */
/*    bgl_get_exitd_stamp ...                                          */
/*---------------------------------------------------------------------*/
obj_t
bgl_get_exitd_stamp() {
   exitd_stamp = BINT( CINT( exitd_stamp ) + 1 );
   return exitd_stamp;
}

/*---------------------------------------------------------------------*/
/*    static obj_t                                                     */
/*    get_current_output_port ...                                      */
/*---------------------------------------------------------------------*/
static obj_t
get_current_output_port() {
   return current_output_port;
}

/*---------------------------------------------------------------------*/
/*    static obj_t                                                     */
/*    set_current_output_port ...                                      */
/*---------------------------------------------------------------------*/
static obj_t
set_current_output_port( obj_t val ) {
   current_output_port = val;
   return BUNSPEC;
}

/*---------------------------------------------------------------------*/
/*    static obj_t                                                     */
/*    get_current_error_port ...                                       */
/*---------------------------------------------------------------------*/
static obj_t
get_current_error_port() {
   return current_error_port;
}

/*---------------------------------------------------------------------*/
/*    static obj_t                                                     */
/*    set_current_error_port ...                                       */
/*---------------------------------------------------------------------*/
static obj_t
set_current_error_port( obj_t val ) {
   current_error_port = val;
   return BUNSPEC;
}

/*---------------------------------------------------------------------*/
/*    static obj_t                                                     */
/*    get_current_input_port ...                                       */
/*---------------------------------------------------------------------*/
static obj_t
get_current_input_port() {
   return current_input_port;
}

/*---------------------------------------------------------------------*/
/*    static obj_t                                                     */
/*    set_current_input_port ...                                       */
/*---------------------------------------------------------------------*/
static obj_t
set_current_input_port( obj_t val ) {
   current_input_port = val;
   return BUNSPEC;
}

/*---------------------------------------------------------------------*/
/*    static struct befored *                                          */
/*    get_befored_top ...                                              */
/*---------------------------------------------------------------------*/
static struct befored *
get_befored_top() {
   return befored_top;
}

/*---------------------------------------------------------------------*/
/*    static void                                                      */
/*    set_befored_top ...                                              */
/*---------------------------------------------------------------------*/
static void
set_befored_top( struct befored *val ) {
   befored_top = val;
}

/*---------------------------------------------------------------------*/
/*    static int                                                       */
/*    get_mvalues_number ...                                           */
/*---------------------------------------------------------------------*/
static int
get_mvalues_number() {
   return mvalues_number;
}

/*---------------------------------------------------------------------*/
/*    static int                                                       */
/*    set_mvalues_number ...                                           */
/*---------------------------------------------------------------------*/
static int
set_mvalues_number( int val ) {
   mvalues_number = val;
   return val;
}

/*---------------------------------------------------------------------*/
/*    static obj_t                                                     */
/*    get_mvalues_val ...                                              */
/*---------------------------------------------------------------------*/
static obj_t
get_mvalues_val( int o ) {
   return mvalues[ o ];
}

/*---------------------------------------------------------------------*/
/*    static obj_t                                                     */
/*    set_mvalues_val ...                                              */
/*---------------------------------------------------------------------*/
static obj_t
set_mvalues_val( int o, obj_t val ) {
   mvalues[ o ] = val;
   return BUNSPEC;
}

/*---------------------------------------------------------------------*/
/*    A list of user variable that can be changed for someone          */
/*    "hooking" on the standard Bigloo library (see for instance       */
/*    the fthread implementation).                                     */
/*---------------------------------------------------------------------*/
char *(*bgl_get_stack_bottom)() = &get_stack_bottom;
void (*bgl_set_stack_bottom)( char * ) = &set_stack_bottom;

obj_t (*bgl_get_exitd_top)() = &get_exitd_top;
obj_t (*bgl_set_exitd_top)( obj_t ) = &set_exitd_top;
struct befored *(*bgl_get_befored_top)() = &get_befored_top;
void (*bgl_set_befored_top)( struct befored * ) = &set_befored_top;

obj_t (*bgl_get_current_output_port)() = &get_current_output_port;
obj_t (*bgl_set_current_output_port)( obj_t ) = &set_current_output_port;

obj_t (*bgl_get_current_error_port)() = &get_current_error_port;
obj_t (*bgl_set_current_error_port)( obj_t ) = &set_current_error_port;

obj_t (*bgl_get_current_input_port)() = &get_current_input_port;
obj_t (*bgl_set_current_input_port)( obj_t ) = &set_current_input_port;

int (*bgl_get_mvalues_number)() = &get_mvalues_number;
int (*bgl_set_mvalues_number)( int ) = &set_mvalues_number;

obj_t (*bgl_get_mvalues_val)( int ) = &get_mvalues_val;
obj_t (*bgl_set_mvalues_val)( int, obj_t ) = &set_mvalues_val;

